__author__ = 'Martin'

import sys
from lists.my_OOP_stack import stack as Stack
from lists.array_types import array_, copy_left, copy_right
from collections.abc import Iterable

DEFAULT_CAPACITY = 2    # use a low value for testing


class List(Stack, Iterable):

    def __init__(self, data=None):
        if isinstance(data, Iterable):
            data_len = len(data)
            self.capacity = data_len * 3 // 2 + 1
            super().__init__(self.capacity)
            copy_left(data, 0, self.array, 0, data_len)
            self.SP = data_len
        else:
            if data == None or data < DEFAULT_CAPACITY:
                self.capacity = DEFAULT_CAPACITY
            else:
                self.capacity = data
            super().__init__(self.capacity)

        self.max_size = sys.maxsize         # set inherited max_size to a very large number

    def _check_capacity(self):
        if self.SP < self.capacity - 1:     # leave a spare element for a guard
            return
        self.capacity = self.capacity * 3 // 2 + 1
        self._new_array()

    def _new_array(self):
        new_array = array_(self.capacity)
        copy_left(self.array, 0, new_array, 0, self.size())
        self.array = new_array

    def push(self, item):
        self._check_capacity()
        super().push(item)

    def insert_at(self, index, item):
        if index > self.size():
            raise IndexError('insert index out of range: ' + str(index))
        self._check_capacity()
        to_move = self.SP - index
        array = self.array
        copy_right(array, index, array, index+1, to_move)
        array[index] = item
        self.SP += 1

    def delete_at(self, index):
        array = self.array
        deleted = array[index]
        to_move = self.SP - index - 1   # one less than insert - not moving the deletion
        copy_left(array, index+1, array, index, to_move)
        self.SP -= 1
        return deleted

    def add_guard(self, item):
        self.array[self.SP] = item

    # the following __methods__ allow a List to be classed as an Iterable
    # and so duplicated by List(another_List). They can be ignored for the present

    def __len__(self):
        return self.size()

    def __getitem__(self, index):
        return self.array[index]

    def __iter__(self):
        ix = 0
        size = self.size()
        while ix < size:
            yield self.array[ix]


if __name__ == '__main__':

    t = List(0)
    for p in range(0, 10):  # i.e. 0..9
        t.push(p)
    print(t.array)
    t.delete_at(0)
    print(t.array)
    print(t)
    t.push(4)
    print(t.array)
    print(t, '\n')

    t = List('abcd')
    print(t.array)
    print(t, '\n')

    t = List([])
    print('t:', t.array)
    print(t)
    t.add_guard(99)
    print(t.array)
    t.insert_at(0, 42)
    print(t.array)
    print(t)
    t.add_guard(99)
    print(t.array, '\n')

    t = List([7, 11, 13])
    print('t:', t.array)
    print(t)
    print(t[1:3])

    u = List(t)
    print('u:', u.array)
    print(u)




    # t = List()
    # t.push(0)
    # print(t.array)
    # t.push(1)
    # print(t.array)
    # for p in range(2, 27): # i.e. 2..26
    #     t.push(p)
    # print(t.array)
    # t.add_guard(27)
    # print(t.array)
    # t.insert_at(0, 99)
    # print(t.array)
    # t.insert_at(27, 27)
    # print(t.array)
    # print(t.delete_at(0))
    # print(t.array)
    # t.push(33)
    # print(t.array)
    # print(t.delete_at(27))
    # print(t.array)
    # print(t.SP)
