__author__ = 'Martin'

from lists.list import List
from lists.array_types import array_, copy_right

"""
    A reduced increase in memory is required in a subclass of List but only for insertions
    Any increase due to pushing items is to remain unchanged
    Here a list using a 'static' __check_capacity() is first defined
"""

class List__(List):

    def __check_capacity(self):         # modified to 'static' method
        if self.SP < self.capacity - 1:
            return
        self.capacity = self.capacity * 3 // 2
        self._new_array()

    def _insert_item(self, index, item):
        to_move = self.SP - index
        array = self.array
        copy_right(array, index, array, index+1, to_move)
        array[index] = item
        self.SP += 1

    def push(self, item):               # modified to call __check_capacity()
        self.__check_capacity()
        super().push(item)

    def insert_at(self, index, item):   # modified to call __check_capacity()
        if index > self.size():
            raise IndexError('insert index out of range: ' + str(index))
        self.__check_capacity()
        self._insert_item(index, item)



class ModList__(List__):

    def __check_capacity(self):         # does not override List__.__check_capacity() so
        if self.SP < self.capacity - 1:
            return
        self.capacity = self.capacity + 2
        self._new_array()

    def insert_at(self, index, item):   # must be overridden to call the new __check_capacity()
        if index > self.size():
            raise IndexError('insert index out of range: ' + str(index))
        self.__check_capacity()
        self._insert_item(index, item)

if __name__ == '__main__':
    lst = ModList__()
    lst.push(1)
    print(lst)
    lst.insert_at(0, 0)
    print(lst)
    lst.insert_at(lst.size(), 2)
    print(lst)
    lst.push(4)
    print(lst)
    lst.insert_at(3, 3)
    print(lst)